import { Column, CreateDateColumn, Entity, PrimaryColumn, PrimaryGeneratedColumn, UpdateDateColumn } from 'typeorm';
import { URL } from 'url';

@Entity('users')
export class User {

    @PrimaryGeneratedColumn({type: 'int4'})
    id: number;
    @Column('varchar', { length: 255,  unique: true, nullable: false })
    username: string;
    @Column('varchar', { length: 255,  nullable: true })
    name?: string;
    @Column('varchar', { length: 255,  nullable: false })
    email: string;
    @Column('varchar', { length: 255,  nullable: false })
    password_hash: string;
    @Column('varchar', { length: 255,  nullable: false })
    salt: string;
    @Column('boolean', { nullable: false })
    active: boolean;
    @Column('boolean', { nullable: false })
    admin: boolean;
    @Column('varchar', { length: 255,  nullable: true })
    avatar?: string;
    @Column('varchar', { length: 255,  nullable: false })
    locale: string;
    @Column('varchar', { length: 255,  nullable: false })
    registration_ip_address: string;
    @Column('varchar', { length: 255,  nullable: false })
    ip_address: string;
    @CreateDateColumn({ nullable: false })
    created_at: Date;
    @UpdateDateColumn({ nullable: false })
    updated_at: Date;

    constructor(props: UserCreate) {
        Object.assign(this, props);
    }

    toJSON() {
        // 这儿是 TypeScript 缺个功能，this 不支持直接用 ... 展开，这里强制类型转换一下。
        // https://github.com/Microsoft/TypeScript/issues/10727
        return { ...<Object>this.cleanSensitiveData(), avatar: this.avatarURL() };
    }

    avatarURL() {
        const avatar = this.avatarURLPure();
        if (avatar) {
            return avatar;
        } else {
            return 'https://cdn02.moecube.com:444/accounts/default_avatar.jpg';
        }
    }

    avatarURLPure() {
        if (this.avatar) {
            let url: URL;
            if (this.avatar.substring(0, 16) == '/uploads/default') {
                url = new URL(this.avatar, 'https://ygobbs.com');
            } else {
                url = new URL(this.avatar, 'https://cdn02.moecube.com:444');
            }
            return url.toString().replace('https://cdn01.moecube.com', 'https://cdn02.moecube.com:444');
        }
        return undefined;
    }

    cleanSensitiveData() {
        // this.email = '_masked';
        if (!this.name) {
            delete this.name;
        }
        if (this.avatar) {
            this.avatar = this.avatar.replace('https://cdn01.moecube.com', 'https://cdn02.moecube.com:444');
        }
        this.password_hash = '_masked';
        this.salt = '_masked';
        this.registration_ip_address = '_masked';
        this.ip_address = '_masked';
        return this;
    }
}

interface UserCreate {
    username: string;
    email: string;
    password_hash: string;
    salt: string;
    active: boolean;
    admin: boolean;
    locale: string;
    registration_ip_address: string;
    ip_address: string;
}

@Entity('tokens')
export class Token {

    @PrimaryColumn('varchar', { length: 255 })
    key: string;
    @Column('varchar', { length: 255,  nullable: false })
    user_id: number;
    @Column('varchar', { length: 255,  nullable: false })
    type: string;
    @Column('varchar', { length: 255,  nullable: false })
    data: string;

    constructor(props: Token) {
        Object.assign(this, props);
    }
}


export class SignIn {

    account: string;
    password: string;

    constructor(props: SignIn) {
        Object.assign(this, props);
    }
}


export interface SignUp {

    username: string;
    name: string;
    password: string;
    email: string;

}

@Entity('username_change_history')
export class UserNameChangeHistory {

    @PrimaryGeneratedColumn({type: 'int4'})
    id: number;
    @Column('int4')
    userid: number;
    @Column('varchar', { length: 255 })
    old_username: string;
    @Column('varchar', { length: 255 })
    new_username: string;
    @UpdateDateColumn()
    change_time: Date;

    constructor(old_username: string, new_username: string, userid: number) {
        this.old_username = old_username;
        this.new_username = new_username;
        this.userid = userid;
        this.change_time = new Date();
    }

}
